package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.entity.IntroduceDetail;
import com.diagbot.entity.IntroduceInfo;
import com.diagbot.entity.IntroduceMap;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.IntroduceDetailServiceImpl;
import com.diagbot.service.impl.IntroduceInfoServiceImpl;
import com.diagbot.service.impl.IntroduceMapServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.IntroduceDetailVO;
import com.diagbot.vo.IntroduceMapVO;
import com.diagbot.vo.IntroducePageVO;
import com.diagbot.vo.IntroduceVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/16 14:30
 */
@Component
public class IntroduceInfoFacade extends IntroduceInfoServiceImpl {
    @Autowired
    IntroduceMapFacade introduceMapFacade;
    @Autowired
    IntroduceDetailFacade introduceDetailFacade;
    @Autowired
    IntroduceDetailServiceImpl introduceDetailServiceImpl;
    @Autowired
    IntroduceMapServiceImpl introduceMapServiceImpl;

    /**
     * 保存提示信息（新增or修改）
     *
     * @param introduceVO
     * @return
     */
    public Boolean saveIntroduce(IntroduceVO introduceVO) {
        IntroduceInfo introduceInfo = new IntroduceInfo();
        if (!(introduceVO.getId() == null)) {
            introduceInfo = this.getById(introduceVO.getId());
            introduceInfo.setModifier(UserUtils.getCurrentPrincipleID());
            introduceInfo.setGmtModified(DateUtil.now());
        } else {
            introduceInfo.setCreator(UserUtils.getCurrentPrincipleID());
            introduceInfo.setGmtCreate(DateUtil.now());
        }
        introduceInfo.setName(introduceVO.getName());
        introduceInfo.setRemark(introduceVO.getRemark());

        //更新提示信息
        this.saveOrUpdate(introduceInfo);

        //明细信息不更新，每次都删除重新插入
        //删除已有明细，逻辑删除
        if (!(introduceInfo.getId() == null)) {
            UpdateWrapper<IntroduceDetail> detailUpdateWrapper = new UpdateWrapper<>();
            detailUpdateWrapper.eq("introduce_id", introduceInfo.getId()).
                    eq("is_deleted", IsDeleteEnum.N.getKey()).
                    set("is_deleted", IsDeleteEnum.Y.getKey()).
                    set("modifier", UserUtils.getCurrentPrincipleID()).
                    set("gmt_modified", DateUtil.now());
            introduceDetailFacade.update(new IntroduceDetail(), detailUpdateWrapper);
        }
        List<IntroduceDetail> introduceDetailList = Lists.newArrayList();
        for (IntroduceDetailVO detailVO : introduceVO.getDetailVOList()) {
            IntroduceDetail detail = new IntroduceDetail();
            detail.setIntroduceId(introduceInfo.getId());
            detail.setCreator(UserUtils.getCurrentPrincipleID());
            detail.setGmtCreate(DateUtil.now());
            detail.setContent(detailVO.getContent());
            detail.setText(detailVO.getText());
            detail.setTitle(detailVO.getTitle());
            detail.setOrderNo(detailVO.getOrderNo());
            detail.setPosition(detailVO.getPosition());
            detail.setIsReason(detailVO.getIsReason());
            introduceDetailList.add(detail);
        }
        //插入新的明细记录
        introduceDetailServiceImpl.saveBatch(introduceDetailList);

        //更新映射关系
        //删除已有映射关系
        UpdateWrapper<IntroduceMap> introduceMapUpdateWrapper = new UpdateWrapper<>();
        introduceMapUpdateWrapper.eq("introduce_id", introduceInfo.getId()).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        introduceMapFacade.update(new IntroduceMap(), introduceMapUpdateWrapper);
        //插入新的映射关系
        List<IntroduceMap> introduceMapList = Lists.newLinkedList();
        for (IntroduceMapVO introduceMapVO : introduceVO.getMapVOList()) {
            IntroduceMap introduceMap = new IntroduceMap();
            BeanUtil.copyProperties(introduceMapVO, introduceMap);
            introduceMap.setIntroduceId(introduceInfo.getId());
            introduceMapList.add(introduceMap);
        }
        introduceMapServiceImpl.saveBatch(introduceMapList);
        return true;
    }

    /**
     * 单条删除提示信息 逻辑删除
     *
     * @param id
     * @return
     */
    public Boolean deleteRecord(Long id) {
        //删除已有映射关系
        UpdateWrapper<IntroduceMap> introduceMapUpdateWrapper = new UpdateWrapper<>();
        introduceMapUpdateWrapper.eq("introduce_id", id).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        introduceMapFacade.update(new IntroduceMap(), introduceMapUpdateWrapper);
        //删除明细
        UpdateWrapper<IntroduceDetail> introduceDetailUpdateWrapper = new UpdateWrapper<>();
        introduceDetailUpdateWrapper.eq("introduce_id", id).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        introduceDetailFacade.update(new IntroduceDetail(), introduceDetailUpdateWrapper);
        //删除提示信息
        UpdateWrapper<IntroduceInfo> introduceInfoUpdateWrapper = new UpdateWrapper<>();
        introduceInfoUpdateWrapper.eq("id", id).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        this.update(new IntroduceInfo(), introduceInfoUpdateWrapper);
        return true;
    }

    /**
     * 批量删除提示信息 逻辑删除
     *
     * @param ids
     * @return
     */
    public Boolean deleteRecords(Long[] ids) {
        //删除已有映射关系
        UpdateWrapper<IntroduceMap> introduceMapUpdateWrapper = new UpdateWrapper<>();
        introduceMapUpdateWrapper.in("introduce_id", ids).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        introduceMapFacade.update(new IntroduceMap(), introduceMapUpdateWrapper);
        //删除明细
        UpdateWrapper<IntroduceDetail> introduceDetailUpdateWrapper = new UpdateWrapper<>();
        introduceDetailUpdateWrapper.in("introduce_id", ids).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        introduceDetailFacade.update(new IntroduceDetail(), introduceDetailUpdateWrapper);
        //删除提示信息
        UpdateWrapper<IntroduceInfo> introduceInfoUpdateWrapper = new UpdateWrapper<>();
        introduceInfoUpdateWrapper.in("id", ids).
                eq("is_deleted", IsDeleteEnum.N.getKey()).
                set("is_deleted", IsDeleteEnum.Y.getKey()).
                set("modifier", UserUtils.getCurrentPrincipleID()).
                set("gmt_modified", DateUtil.now());
        this.update(new IntroduceInfo(), introduceInfoUpdateWrapper);
        return true;
    }

    /**
     * 分页查询提示信息，可带等于条件
     *
     * @param introducePageVO
     * @return
     */
    public IPage<IntroduceInfo> getPageByMap(IntroducePageVO introducePageVO) {
        QueryWrapper<IntroduceInfo> introduceInfoQueryWrapper = new QueryWrapper<>();
        for (Map.Entry<String, Object> entry : introducePageVO.getMap().entrySet()) {
            introduceInfoQueryWrapper.eq(entry.getKey(), entry.getValue());
        }
        IPage<IntroduceInfo> introduceInfoIPage = this.page(introducePageVO, introduceInfoQueryWrapper);
        return introduceInfoIPage;
    }

    /**
     * 带条件查询
     *
     * @param map
     * @return
     */
    public List<IntroduceInfo> getByMap(Map<String, Object> map) {
        QueryWrapper<IntroduceInfo> introduceInfoQueryWrapper = new QueryWrapper<>();
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            introduceInfoQueryWrapper.eq(entry.getKey(), entry.getValue());
        }
        return this.list(introduceInfoQueryWrapper);
    }
}