package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.QuestionIndexDTO;
import com.diagbot.entity.QuestionInfo;
import com.diagbot.entity.wrapper.QuestionWrapper;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.QuestionInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.QuestionSaveVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2018/11/23 11:37
 */
@Component
public class QuestionInfoFacade extends QuestionInfoServiceImpl {

    @Autowired
    QuestionDetailFacade questionDetailFacade;
    @Autowired
    QuestionMappingFacade questionMappingFacade;
    @Autowired
    ModuleDetailFacade moduleDetailFacade;


    /**
     * 标签保存
     *
     * @param questionSaveVO
     */
    public void saveOrUpdate(QuestionSaveVO questionSaveVO) {

//        String person = UserUtils.getCurrentPrincipleID(); //TODO person
        String person = "default";
        QuestionWrapper questionWrapper = questionSaveVO.getQuestionWrapper();
        //1、主表信息
        QuestionInfo questionInfo = new QuestionInfo();
        String saveOrUpdate = "save"; //新增或修改的标识
        if(questionWrapper.getId() != null) {
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("id", questionWrapper.getId());
            queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
            questionInfo = getOne(queryWrapper);
            if(questionInfo == null) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                        "标签不存在");
            }
            saveOrUpdate = "update";
        }
        BeanUtil.copyProperties(questionWrapper, questionInfo);
        //标签type、tagName唯一
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("tag_name", questionInfo.getTagName());
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
        queryWrapper.eq("type", questionInfo.getType());
        queryWrapper.ne("id", questionInfo.getId() == null ? 0 : questionInfo.getId());
        List<QuestionInfo> questionInfoList = this.list(queryWrapper);
        if(questionInfoList.size() > 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "标签名已存在");
        }
        if("save".equals(saveOrUpdate)) {
            questionInfo.setCreator(person); //创建人
            questionInfo.setGmtCreate(DateUtil.now());//创建时间
        } else {
            questionInfo.setGmtModified(DateUtil.now());//修改时间
            questionInfo.setModifier(person);//修改人
        }
        this.saveOrUpdate(questionInfo);


        //明细信息

        //映射信息

        //同义词信息

    }


    /**
     * 根据id删除标签
     *
     * @param ids
     * @return
     */
    public Boolean deleteByIdsFac(String ids) {

        if(StringUtil.isEmpty(ids)) {
            return true;
        }

        Map paramMap = new HashMap<>();
        paramMap.put("delete", IsDeleteEnum.Y.getKey());
        paramMap.put("ids", Arrays.asList(ids.split(",")));
        paramMap.put("gmtModified", DateUtil.now());
        paramMap.put("modifier", UserUtils.getCurrentPrincipleID());

        //删除自身
        this.deleteByIds(paramMap);

        //删除明细
        questionDetailFacade.deleteByQuestionIdFac(paramMap);

        //删除映射关系
        questionMappingFacade.deleteByIdsFac(paramMap);

        //删除模板中的标签
        moduleDetailFacade.deleteByQuestionIdFac(paramMap);
        return true;
    }


    /**
     * 检索
     *
     * @param questionIndexDTO
     * @return
     */
    public List<QuestionInfo> indexFac(QuestionIndexDTO questionIndexDTO) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("isDeleted", IsDeleteEnum.N.getKey());
        paramMap.put("age", questionIndexDTO.getAge());
        paramMap.put("sexType", questionIndexDTO.getSexType());
        paramMap.put("tagName", questionIndexDTO.getTagName());
        paramMap.put("type", questionIndexDTO.getType());
        return this.index(paramMap);
    }

    /**
     * 根据标签名获取标签信息
     * @param questionInfo
     * @return
     */
    public QuestionInfo getQuestionInfos(QuestionInfo questionInfo){
        QueryWrapper<QuestionInfo> queryWrapper = new QueryWrapper<QuestionInfo>();
        queryWrapper.eq("is_deleted", "N");
        queryWrapper.eq("type", 1);
        queryWrapper.eq("name",questionInfo.getName());
        return this.getOne(queryWrapper);
    }
}
