package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.DeptInfo;
import com.diagbot.entity.QuestionUsual;
import com.diagbot.facade.DeptInfoFacade;
import com.diagbot.facade.QuestionUsualFacade;
import com.diagbot.vo.QuestionUsualVO;
import com.diagbot.vo.UpdateQuetionUsualVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 常用标签表 前端控制器
 * </p>
 *
 * @author wangyu
 * @since 2018-12-03
 */
@RestController
@RequestMapping("/questionUsual")
@SuppressWarnings("unchecked")
@Api(value = "常用标签维护相关API", tags = { "常用标签维护相关API" })
public class QuestionUsualController {

    @Autowired
    private QuestionUsualFacade questionUsualFacade;
    @Autowired
    private DeptInfoFacade deptInfoFacade;

    @ApiOperation(value = "常用标签维护——添加[by:wangyu]",
            notes = "deptCode: 科室编号,必填<br>" +
                    "hospitalCode: 医院编号，必填<br>" +
                    "questionName: 症状名称，必填")
    @PostMapping("/addQuestionUsual")
    @SysLogger("addQuestionUsual")
    @Transactional
    public RespDTO<Boolean> addQuestionUsual(@Valid @RequestBody QuestionUsualVO questionUsualVO) {
        Boolean data = questionUsualFacade.addQuestionUsual(questionUsualVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "常用标签维护——修改[by:wangyu]",
            notes = "id: 常用标签id,必填<br>" +
                    "orderNo: 排序号<br>" +
                    "")
    @PostMapping("/updateQuestionUsual")
    @SysLogger("updateQuestionUsual")
    @Transactional
    public RespDTO<Boolean> updateQuestionUsual(@Valid @RequestBody UpdateQuetionUsualVO updateQuetionUsualVO) {
        Boolean data = questionUsualFacade.updateQuestionUsual(updateQuetionUsualVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "常用标签维护——删除[by:wangyu]",
            notes = "id: 常用标签id,必填<br>")
    @PostMapping("/deleteQuestionUsual")
    @SysLogger("deleteQuestionUsual")
    @Transactional
    public RespDTO<Boolean> deleteQuestionUsual(@Valid @RequestBody UpdateQuetionUsualVO updateQuetionUsualVO) {
        Boolean data = questionUsualFacade.deleteQuestionUsual(updateQuetionUsualVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "常用标签维护——查询[by:wangyu]",
            notes = "id: 常用标签id,必填<br>")
    @PostMapping("/getQuestionUsual")
    @SysLogger("getQuestionUsual")
    public RespDTO<IPage<QuestionUsual>> getQuestionUsual() {
        IPage<QuestionUsual> data = questionUsualFacade.getQuestionUsual();
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "常用标签维护——获取科室名称[by:wangyu]",
            notes = "")
    @PostMapping("/getDeptName")
    @SysLogger("getDeptName")
    public RespDTO<List<DeptInfo>> getDeptName() {
        List<DeptInfo> data = deptInfoFacade.getAllDeptInfo();
        return RespDTO.onSuc(data);
    }
}
